import java.applet.Applet;
import java.util.*;
import java.awt.*;
import java.lang.*;
import java.io.*;
import java.net.URL;

public class BarGraph extends Applet {

    private boolean   debug_B   = true;
    private boolean   orient_B  = true;  // Verticle
    private boolean   axisMax_B = true;  // maximum of data used
    private int       plotType  = 0;     // 0 - side by side
                                         // 1 - add to
                                         // 2 - 3-dimension
    private int       NoSets    = 1;     // Number of sets of data
    private int       NoLines   = 1;     // Total Number of bars to plot
    private int       NoColumns = 3;     // always (NoSets+NoLines+1)
    private double    Unit      = 1.0;   // Scaling factor for plot

    private Font      TitleFont,   LabelFont;
    private FontMetrics TitleFM,   LabelFM;
    private String    Title,       TitleFontN,   LabelFontN;
    private int       TitleSize,   TitleStyle,   TitleHeight, TitleWidth;
    private int       LabelSize,   LabelStyle,   LabelHeight, LabelWidth;

    private Color     colorBG,     colorFGT,   colorFG;

    private String    sLabel[]    = { " ","","","","","","","","","","","" }; 
    private int       Set[]       = {   1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    private double    Value[]     = { 1.0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    private int       sLabelWidth[] = {   1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
    private Color     Color1[];

    private int       ii[],  sLW1[];
    private double    V1[];
    private Color     C1[];
    private String    S1[];

    private double    minBarWidth,  BarWidth,    BarWidth1,  BarHeight1;
    private double    maxValue,     maxValue1;
    private double    maxLblWidth = 0.0;

    private int       Width1,   Height1;
    private double    dWidth1,  dHeight1;



    // ------------------------------------------------------------------
    
    public synchronized void init() {

        colorBG     = getColorParam("BGcolor", "white");
        setBackground(colorBG);
        colorFG     = getColorParam("FGcolor ", "red");
        setForeground(colorFG);

//      setFont(getFontParam("TitleFONT","Helvetica","TitleSTYLE",1,"TitleSIZE",12));
        TitleFontN  = getStringParam("TitleFONT","Helvetica");
        TitleSize   = getIntParam("TitleSIZE",12,6,48);
        TitleStyle  = getIntParam("TitleSTYLE",0,0,3);
        setFont(new Font(TitleFontN, TitleStyle, TitleSize));
        TitleFont   = getFont();
        TitleFM     = getFontMetrics(getFont());

        Title       = getStringParam("TITLE","Bar Chart");
        TitleHeight = TitleFM.getHeight();
        TitleWidth  = TitleFM.stringWidth(Title);
        colorFGT    = getColorParam("FGcolorT", "magenta");

        NoLines     = getIntParam("PLOTS",1,1,10);
        NoSets      = getIntParam("SETS",1,1,10);

        plotType    = getIntParam("plotTYPE",0,0,1);  // 0 = side by side
                                                      // 1 = add to
                                                      // 2 = 3-d in sets (not implemented)
        NoColumns   = NoSets + NoLines + 1;

        Unit        = getDoubleParam("UNIT",1.0,1.0,200.0);  

        Color1      = new Color[12];
//      setFont(getFontParam("LabelFONT","Helvetica","LabelSTYLE",0,"LabelSIZE",9));
        LabelFontN  = getStringParam("LabelFONT","Helvetica");
        LabelSize   = getIntParam("LabelSIZE",12,6,48);
        LabelStyle  = getIntParam("LabelSTYLE",0,0,3);
        setFont(new Font(LabelFontN, LabelStyle, LabelSize));
        LabelFont   = getFont();
        LabelFM     = getFontMetrics(getFont());
        LabelHeight = LabelFM.getHeight();

        sLW1 = new int[24];
        V1   = new double[24];
        C1   = new Color[24];
        S1   = new String[24];

        int iCk  = 0;
        int iii  = 0;
        maxValue = 0.0;
        for (int i = 0; i < NoLines; i++) {
            S1[iii]   = "";
            C1[iii]   = colorBG;
            V1[iii]   = 0.0;
            sLW1[iii] = 0;

            String setNumber = "SET" + i;
            Set[i]           = getIntParam(setNumber,1,1,12);

            String plotName  = "PLOT" + i;
            sLabel[i]        = getStringParam(plotName," ");
            sLabelWidth[i]   = LabelFM.stringWidth(sLabel[i]);
            maxLblWidth      = Math.max(sLabelWidth[i],maxLblWidth);

            String Value1    = "VALUE" + i;
            Value[i]         = getDoubleParam(Value1, 1.0, 0.0, 1000.0);
            maxValue1        = Math.max(Value[i],maxValue);
            maxValue         = maxValue1;

            String ColorName = "COLOR" + i;
            Color1[i]        = getColorParam(ColorName,"red");

            if (Set[i] > iCk) {
                iii += 1;
                iCk  = Set[i];
            }
            S1[iii]   = sLabel[i];
            C1[iii]   = Color1[i];
            V1[iii]   = Value[i];
            sLW1[iii] = sLabelWidth[i];
            iii += 1;
        }

        S1[NoColumns]   =  "";
        C1[NoColumns]   = colorBG;
        V1[NoColumns]   = 0.0;
        sLW1[NoColumns] = 0;

        axisMax_B = getBoolParam("MaxFromData", "true");    // true = use data
        if (!axisMax_B) maxValue   = getDoubleParam("maxVALUE", 1.0, 0.0, 1000.0);
        else            maxValue  *= 1.01;

        int maxV    = (int)maxValue;

        orient_B    = getBoolParam("ORIENTATION", "true");    // default is true = Verticle
        minBarWidth = maxLblWidth + 10;

        Width1      = size().width;
        Height1     = size().height - (TitleHeight + 10) - (LabelHeight + 3 );
        dWidth1     = (double)Width1;
        dHeight1    = (double)Height1;

        if (orient_B) {
            BarWidth1   = Math.max(dWidth1/((double)NoColumns),minBarWidth);
            BarHeight1  = dHeight1;
        } else {
            minBarWidth = (double) LabelHeight;
            double Tmp  = (dHeight1 + (LabelHeight + 3 ))/((double)NoColumns);
            BarWidth1   = Math.max(Tmp,minBarWidth);
            BarHeight1  = dWidth1 - maxLblWidth - 2.0 - 2.0*((double)TitleHeight);
        }
    }

    // ------------------------------------------------------------------

    public synchronized void paint(Graphics g) {

	int Xpt, Ypt;
        double dXpt, dYpt;

	g.setColor(colorFGT);
        g.setFont(TitleFont);
	g.drawString(Title, Math.max((size().width - TitleWidth)/2, 0),
		     size().height - TitleFM.getDescent()); 	// Centered Title

        g.setFont(LabelFont);

	for (int i=0; i < NoColumns; i++) {
            int iValue    = (int)(Unit*V1[i]);
            int iBarWidth = (int)BarWidth1;
            int imaxValue = (int)maxValue;
            int imaxLblWidth = (int)maxLblWidth;

            if (orient_B) {
                dXpt  = ((double)i)*BarWidth1;
                Xpt   = (int) dXpt;
                dYpt  = dHeight1 - ((dHeight1 - 20.0)*V1[i]/maxValue);
                Ypt   = (int) dYpt;
                System.out.println("i, Xpt, Ypt ,iValue = "+i+"   "+Xpt+"   "+Ypt+"   "+iValue);

                g.setColor(Color.black);
                if (iValue != 0) {
                    g.drawString(S1[i], Xpt + (iBarWidth - sLW1[i])/2, 
                        size().height - LabelFont.getSize() - LabelFM.getDescent() - 18);

                    if (C1[i] == Color.black) g.setColor(Color.gray);
                    g.drawRect(Xpt, Ypt, iBarWidth, 
                        (int)((dHeight1 - 20.0)*V1[i]/maxValue));
                    g.fillRect(Xpt + 3, Ypt - 2, iBarWidth, 
                        (int)((dHeight1 - 20.0)*V1[i]/maxValue) + 2);

		    g.setColor(C1[i]);
		    g.fillRect(Xpt + 1, Ypt + 1, iBarWidth - 1, 
                        (int)((dHeight1 - 20.0)*V1[i]/maxValue) - 1);

                    g.setColor(Color.black);
		    g.drawString("    "+V1[i], Xpt, Ypt - LabelFM.getDescent() - 2);
                } else 
                    g.drawRect(Xpt, Ypt - 2 + (int)((dHeight1 - 20.0)*V1[i]/maxValue),
                        iBarWidth, 0);

            } else {

                dYpt  = ((double)i)*BarWidth1;
                Ypt   = (int) dYpt;
                dXpt  = ((double)TitleHeight);
                Xpt   = (int) dXpt;

                g.setColor(Color.black);
                if (i == (NoColumns - 1)) g.drawRect(0, Ypt + (int)(BarWidth1), Width1, 0);
                if (iValue != 0) {
                    g.drawString(S1[i], Xpt, Ypt + ((int)(2.0*BarWidth1/3.0)));

                    if (C1[i] == Color.black) g.setColor(Color.gray);
                    g.drawRect(Xpt + ((int)maxLblWidth) + 2, Ypt,
                        (int)(BarHeight1*V1[i]/maxValue), iBarWidth);
                    g.fillRect(Xpt + ((int)maxLblWidth) + 2 + 2, Ypt + 3,
                        (int)(BarHeight1*V1[i]/maxValue), iBarWidth);

                    g.setColor(C1[i]);
                    g.fillRect(Xpt + ((int)maxLblWidth) + 2 + 1, Ypt + 1,
                        (int)(BarHeight1*V1[i]/maxValue), iBarWidth);

                    g.setColor(Color.black);
		    g.drawString(" "+V1[i], Xpt + ((int)maxLblWidth) + 10, Ypt + ((int)(2.0*BarWidth1/3.0)));
                } else 
                    g.drawRect(Xpt + ((int)maxLblWidth) + 2, Ypt, 0, iBarWidth);
	    } // if orient_B
	}  // end for loop
    }

    // ------------------------ getString -------------------------------

    public String getStringParam(String x, String def_X) {
        String str;
        str = getParameter(x);
        if (str == null) str = def_X;
        return str;
    }

    // ----------------------- getInteger -------------------------------

    public int getIntParam(String i, int def_I, int min_i, int max_i) {
        int i_in;
        try {
            i_in = Integer.parseInt(getParameter(i));
            if ((i_in < min_i)||(i_in > max_i)) throw new Exception();
        } catch (Exception e) {
            System.out.println(i+" invalid; using default value: "+def_I);
            i_in = def_I;
        }	
        return i_in;
    }

    // ------------------------ getColor --------------------------------

    public Color getColorParam(String c, String def_C) {

        Color Color1 = Color.lightGray;      // default color
        if (def_C != null) {
            if      (def_C.equalsIgnoreCase("black"))     Color1 = Color.black;
            else if (def_C.equalsIgnoreCase("blue"))      Color1 = Color.blue;
            else if (def_C.equalsIgnoreCase("cyan"))      Color1 = Color.cyan;
            else if (def_C.equalsIgnoreCase("darkGray"))  Color1 = Color.darkGray;
            else if (def_C.equalsIgnoreCase("gray"))      Color1 = Color.gray;
            else if (def_C.equalsIgnoreCase("green"))     Color1 = Color.green;
            else if (def_C.equalsIgnoreCase("lightGray")) Color1 = Color.lightGray;
            else if (def_C.equalsIgnoreCase("magenta"))   Color1 = Color.magenta;
            else if (def_C.equalsIgnoreCase("orange"))    Color1 = Color.orange;
            else if (def_C.equalsIgnoreCase("pink"))      Color1 = Color.pink;
            else if (def_C.equalsIgnoreCase("red"))       Color1 = Color.red;
            else if (def_C.equalsIgnoreCase("white"))     Color1 = Color.white;
            else if (def_C.equalsIgnoreCase("yellow"))    Color1 = Color.yellow;
        }

        StringTokenizer st;
        String rgbDelim = ":,.";
        String c_str = getParameter(c);
        if (c_str != null) st = new StringTokenizer(c_str, rgbDelim);
        else return Color1;

        if      (c_str.equalsIgnoreCase("black"))     return Color.black;
        else if (c_str.equalsIgnoreCase("blue"))      return Color.blue;
        else if (c_str.equalsIgnoreCase("cyan"))      return Color.cyan;
        else if (c_str.equalsIgnoreCase("darkGray"))  return Color.darkGray;
        else if (c_str.equalsIgnoreCase("gray"))      return Color.gray;
        else if (c_str.equalsIgnoreCase("green"))     return Color.green;
        else if (c_str.equalsIgnoreCase("lightGray")) return Color.lightGray;
        else if (c_str.equalsIgnoreCase("magenta"))   return Color.magenta;
        else if (c_str.equalsIgnoreCase("orange"))    return Color.orange;
        else if (c_str.equalsIgnoreCase("pink"))      return Color.pink;
        else if (c_str.equalsIgnoreCase("red"))       return Color.red;
        else if (c_str.equalsIgnoreCase("white"))     return Color.white;
        else if (c_str.equalsIgnoreCase("yellow"))    return Color.yellow;

        else if (st.countTokens() == 3) {
            Integer r = new Integer(st.nextToken());
            Integer g = new Integer(st.nextToken());
            Integer b = new Integer(st.nextToken());
            Color1    = new Color(r.intValue(), g.intValue(), b.intValue());
            return Color1;
            }
        return Color1;
    }

    // -------------------------- getFont  ------------------------------

    public Font getFontParam(String fnt, String def_Fnt, 
            String Style, int def_Style, String Size, int def_Size) {

        String font_Name  = getStringParam(fnt,   def_Fnt);
        int    font_Style = getIntParam   (Style, def_Style, 0, 3);
        int    font_Size  = getIntParam   (Size,  def_Size, 4, 100);

        return (new Font(font_Name, font_Style, font_Size));
    }

    // ---------------------------- getFloat ----------------------------

    public float getFloatParam(String f, float def_F, float min_f, float max_f) {
        float ff;
        try {
            String f_str = getParameter(f);
            ff = Float.valueOf(f_str).floatValue();
            if ((ff < min_f)||(ff > max_f)) throw new Exception();
        } catch (Exception e) {
                System.out.println(f+" Invalid; using default value: "+def_F);
                ff = def_F;
        }	
        return ff;
    }

    // ---------------------------- getDouble ---------------------------

    public double getDoubleParam(String f, double def_F, double min_f, double max_f) {
        double ff;
        try {
            String f_str = getParameter(f);
            ff = Double.valueOf(f_str).doubleValue();
            if ((ff < min_f)||(ff > max_f)) throw new Exception();
        } catch (Exception e) {
                System.out.println(f+" Invalid; using default value: "+def_F);
                ff = def_F;
        }	
        return ff;
    }

    // ----------------------------- getBool ----------------------------

    public boolean getBoolParam(String b, String def_B) {

        String b_str;
        b_str = getParameter(b);
        if (b_str != null) {
            if (b_str.equalsIgnoreCase("false"))      return false;
            else if (b_str.equalsIgnoreCase("true"))  return true;
            else if (b_str.equalsIgnoreCase("0"))     return false;
            else if (b_str.equalsIgnoreCase("1"))     return true;

            else if (def_B != null) {
                if (def_B.equalsIgnoreCase("false"))  return false;
                else if (def_B.equalsIgnoreCase("0")) return false;
                else                                  return true;
            } else                                    return true;
        } else if (def_B != null) {
            if (def_B.equalsIgnoreCase("false"))      return false;
            else if (def_B.equalsIgnoreCase("0"))     return false;
            else                                      return true;
        }   
        return true;
    }
    // ------------------------------------------------------------------

}

// ======================================================================



/*
            } else {
		Ypt = ((BarWidth + barSpacing) * i) + barSpacing;
		Xpt = maxLabelWidth + 1;
		Xpt += Math.max((size().width - (maxLabelWidth + 1 +
                    TitleFM.stringWidth("" + max) + (max * scale))) / 2, 0);
		g.setColor(Color.black);		
		g.drawString((String)labels[i], Xpt - maxLabelWidth - 1,
                    Ypt + TitleFM.getAscent());
		if (colors[i] == Color.black) g.setColor(Color.gray);
		g.fillRect(Xpt + 3, Ypt + 5, (values[i] * scale), BarWidth);
                g.setColor((Color)(colors[i]));
		g.fillRect(Xpt, Ypt, (values[i] * scale), BarWidth);
		g.drawString("" + values[i], Xpt + (values[i] * scale) + 3,
                    Ypt + TitleFM.getAscent());
	    } // if orient_B

		Xpt = imaxLblWidth + 1;
		Xpt += Math.max((size().width - (imaxLblWidth + 1 +
                    LabelFM.stringWidth("" + imaxValue) + (imaxValue*((int)Unit))))/2, 0);

		g.setColor(Color.black);		
		g.drawString(S1[i], Xpt - imaxLblWidth - 1, Ypt + LabelFM.getAscent());
		if (C1[i] == Color.black) g.setColor(Color.gray);
		g.fillRect(Xpt + 3, Ypt + 5, iValue, iBarWidth);
                g.setColor(C1[i]);
		g.fillRect(Xpt, Ypt, iValue, iBarWidth);
		g.drawString("" + iValue, Xpt + iValue + 3, Ypt + LabelFM.getAscent());
*/